package com.guildasoft.bluechatroom2;

import static com.guildasoft.bluechatroom2.utils.MessageUtilsImpl.MAX_TEXT_SIZE;

import android.os.Bundle;
import android.text.Editable;
import android.text.InputFilter;
import android.text.TextWatcher;
import android.util.Log;
import android.view.Gravity;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.EditText;
import android.widget.FrameLayout;
import android.widget.ImageView;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.fragment.app.Fragment;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import com.google.android.material.snackbar.Snackbar;
import com.guildasoft.bluechatroom2.entity.AppMessage;

import org.jetbrains.annotations.NotNull;

import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

public class ChatFragment extends Fragment implements View.OnClickListener {
    private EditText chatInput;
    private FrameLayout sendButton;
    private CommunicationListener communicationListener;
    private RecyclerView recyclerViewChat;
    private ChatAdapter chatAdapter;
    private List<AppMessage> appMessageList = new ArrayList<>();
    private View globalChatView;
    Snackbar snackbar;


    @Nullable
    @org.jetbrains.annotations.Nullable
    @Override
    public View onCreateView(@NonNull @NotNull LayoutInflater inflater, @Nullable @org.jetbrains.annotations.Nullable ViewGroup container, @Nullable @org.jetbrains.annotations.Nullable Bundle savedInstanceState) {
        View view = LayoutInflater.from(getActivity()).inflate(R.layout.chat_fragment, container, false);
        initViews(view);
        return view;
    }

    private void initViews(View view){
        globalChatView = view;
        chatInput = view.findViewById(R.id.chatInput);
        ImageView chatIcon = view.findViewById(R.id.sendIcon);
        sendButton = view.findViewById(R.id.sendButton);
        recyclerViewChat = view.findViewById(R.id.chatRecyclerView);

        sendButton.setClickable(false);
        sendButton.setEnabled(false);

        LinearLayoutManager llm = new LinearLayoutManager(getActivity());
        llm.setReverseLayout(false);
        llm.setStackFromEnd(true);
        recyclerViewChat.setLayoutManager(llm);

        chatInput.addTextChangedListener(createTextWatcher(chatIcon));
        InputFilter[] FilterArray = new InputFilter[1];
        FilterArray[0] = new InputFilter.LengthFilter(MAX_TEXT_SIZE);
        chatInput.setFilters(FilterArray);

        sendButton.setOnClickListener(this);

        List<AppMessage> messagesCopy = appMessageList;
        chatAdapter = new ChatAdapter(messagesCopy, Objects.requireNonNull(getActivity()));

        recyclerViewChat.setAdapter(chatAdapter);
    }

    private TextWatcher createTextWatcher(ImageView chatIcon){
        return new TextWatcher() {
            @Override
            public void beforeTextChanged(CharSequence charSequence, int i, int i1, int i2) {}
            @Override
            public void onTextChanged(CharSequence charSequence, int i, int i1, int i2) {}
            @Override
            public void afterTextChanged(Editable editable) {
                if(editable.length() > 0) {
                    chatIcon.setImageDrawable(
                            Objects.requireNonNull(getActivity()).getDrawable(R.drawable.ic_send)
                    );
                    sendButton.setClickable(true);
                    sendButton.setEnabled(true);
                }else{
                    chatIcon.setImageDrawable(
                            Objects.requireNonNull(getActivity()).getDrawable(R.drawable.ic_send_depri)
                    );
                    sendButton.setClickable(false);
                    sendButton.setClickable(false);
                }
            }
        };
    }

    @Override
    public void onClick(View view) {
        if(chatInput.getText().length() > 0){
            communicationListener.onCommunication(chatInput.getText().toString());
            chatInput.setText("");
        }
    }

    interface CommunicationListener{
        public void onCommunication(String message);
    }

    public void showSnackEmptyChat(){
        try {
            View coiso = globalChatView.findViewById(R.id.coordenadoSnack);
            snackbar = Snackbar.make(coiso, R.string.empty_chat, 3000);
            View snackView = snackbar.getView();
            snackView.setBackgroundColor(getResources().getColor(R.color.error));
            TextView textView = snackView.findViewById(R.id.snackbar_text);
            textView.setGravity(Gravity.CENTER_HORIZONTAL);
            snackbar.show();
        }catch (Exception e){
            Log.d("TAG", e.getMessage());
        }
    }

    public void communicate(AppMessage appMessage){
        appMessageList.add(appMessage);
        if(Objects.nonNull(getActivity())){
            List<AppMessage> messagesCopy = appMessageList;
            chatAdapter = new ChatAdapter(messagesCopy, Objects.requireNonNull(getActivity()));
            recyclerViewChat.setAdapter(chatAdapter);
            recyclerViewChat.scrollToPosition(recyclerViewChat.getBaseline());
        }
    }

    public void setCommunicationListener(CommunicationListener communicationListener){
        this.communicationListener = communicationListener;
    }
}
